--版本控制：1.6.0 提供Spell Table还原默认值的方法

--[[
	致谢：Akinoso，参考了你的AkiTools很多
]]

local AddonName, Addon = ...

local Config = Addon.Config
local Panel = Addon.Panel
local ScrollFrame = Addon.ScrollFrame
local L = Addon.L

--设置面板初始化
Panel:SetSize(500, 1000)
ScrollFrame.ScrollBar:ClearAllPoints()
ScrollFrame.ScrollBar:SetPoint("TOPLEFT", ScrollFrame, "TOPRIGHT", -20, -20)
ScrollFrame.ScrollBar:SetPoint("BOTTOMLEFT", ScrollFrame, "BOTTOMRIGHT", -20, 20)
ScrollFrame:SetScrollChild(Panel)
ScrollFrame.name = AddonName
InterfaceOptions_AddCategory(ScrollFrame)
--标题
local PanelTitle = Panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLargeLeft")
PanelTitle:SetPoint("TOPLEFT", 16, -16)
PanelTitle:SetText("|cFFFF6633"..AddonName.."|r".." v"..Addon.Version)
--所有组件表
Panel.Controls = {}
--创建唯一命名函数
local UniqueName
do
	local ControlID = 1

	function UniqueName(Name)
		ControlID = ControlID + 1
		return string.format("%s_%s_%02d", AddonName, Name, ControlID)
	end
end
--设置面板确定函数
function ScrollFrame:ConfigOkay()
	for _, Control in pairs(Panel.Controls) do
		Control.SaveValue(Control.CurrentValue)
	end
	local msg = ""
	for k, v in pairs(Addon.RunTime[Config.SelectedSpellType]) do
		if msg == "" then
			msg = Addon.RunTime[Config.SelectedSpellType][k]
		else
			msg = msg .. "\n" .. Addon.RunTime[Config.SelectedSpellType][k]
		end
	end
	Addon.CurrentSpellListText = msg
end
--设置面板回到默认设置函数
function ScrollFrame:ConfigDefault()
	for _, Control in pairs(Panel.Controls) do
		Control.CurrentValue = Control.DefaultValue
		Control.SaveValue(Control.CurrentValue)
	end
end
--设置面板刷新函数
function ScrollFrame:ConfigRefresh()
	for _, Control in pairs(Panel.Controls) do
		Control.CurrentValue = Control.LoadValue()
		Control:UpdateValue()
	end
end
--创建标题函数
function Panel:CreateHeading(text)
	local Title = self:CreateFontString(nil, "ARTWORK", "GameFontNormalLeft")
	Title:SetText(text)
	return Title
end
--创建文本函数
function Panel:CreateText(text)
	local TextBlob = self:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmallLeft")
	TextBlob:SetText(text)
	return TextBlob
end
--创建按钮函数
function Panel:CreateButton(text, RunFunction)
	local Button = CreateFrame("Button", nil, self, "UIPanelButtonTemplate")
	Button:Enable()
	Button:SetSize(50, 26)
	Button:SetText(text)
	Button:SetScript("OnClick", RunFunction)
	return Button
end
--创建选择框函数
function Panel:CreateCheckBox(text, LoadValue, SaveValue, DefaultValue)
	local CheckBox = CreateFrame("CheckButton", UniqueName("CheckButton"), self, "InterfaceOptionsCheckButtonTemplate")

	CheckBox.LoadValue = LoadValue
	CheckBox.SaveValue = SaveValue
	CheckBox.DefaultValue = DefaultValue
	CheckBox.UpdateValue = function(self) self:SetChecked(self.CurrentValue) end
	getglobal(CheckBox:GetName() .. "Text"):SetText(text)
	CheckBox:SetScript("OnClick", function(self) self.CurrentValue = self:GetChecked() end)

	self.Controls[CheckBox:GetName()] = CheckBox
	return CheckBox
end
--下拉菜单点击函数
local function DropDownOnClick(_, DropDown, SelectedValue)
	DropDown.CurrentValue = SelectedValue
	UIDropDownMenu_SetText(DropDown, DropDown.ValueTexts[SelectedValue])
end
--下拉菜单初始化函数
local function DropDownInitialize(frame)
	local info = UIDropDownMenu_CreateInfo()

	for i=1, #frame.ValueList, 2 do
		local k, v = frame.ValueList[i], frame.ValueList[i + 1]
		info.text = v
		info.value = k
		info.checked = frame.CurrentValue == k
		info.func = DropDownOnClick
		info.arg1, info.arg2 = frame, k
		UIDropDownMenu_AddButton(info)
	end
end
--创建下拉菜单函数
function Panel:CreateDropDown(text, ValueList, LoadValue, SaveValue, DefaultValue)
	local DropDown = CreateFrame("Frame", UniqueName("DropDown"), self, "UIDropDownMenuTemplate")

	local Title = self:CreateFontString(nil, "ARTWORK", "GameFontNormalSmallLeft")
	Title:SetText(text)
	Title:SetPoint("BOTTOMLEFT", DropDown, "left", -80, -6)

	DropDown.LoadValue = LoadValue
	DropDown.SaveValue = SaveValue
	DropDown.DefaultValue = DefaultValue
	DropDown.UpdateValue = function(self)
		UIDropDownMenu_SetText(self, self.ValueTexts[self.CurrentValue])
	end

	DropDown.ValueList = ValueList
	DropDown.ValueTexts = {}
	for i=1, #ValueList, 2 do
		local k, v = ValueList[i], ValueList[i + 1]
		DropDown.ValueTexts[k] = v
	end

	DropDown:SetScript("OnShow", function(self)
		UIDropDownMenu_Initialize(self, DropDownInitialize)
	end)

	UIDropDownMenu_JustifyText(DropDown, "LEFT")
	UIDropDownMenu_SetWidth(DropDown, 120)
	UIDropDownMenu_SetButtonWidth(DropDown, 144)

	self.Controls[DropDown:GetName()] = DropDown
	return DropDown
end
--创建滑块函数
function Panel:CreateSliderBar(_, LoadValue, SaveValue, DefaultValue)
	local SliderBar = CreateFrame("Slider", UniqueName("Slider"), self, "OptionsSliderTemplate")

	SliderBar.LoadValue = LoadValue
	SliderBar.SaveValue = SaveValue
	SliderBar.DefaultValue = DefaultValue
	SliderBar.UpdateValue = function(self) self:SetValue(self.CurrentValue) end
	SliderBar:SetScript("OnValueChanged", function(self, v)
		self.CurrentValue = math.floor(self:GetValue())
		self.Text:SetText(SecondsToTime(v))
	end)

	self.Controls[SliderBar:GetName()] = SliderBar
	return SliderBar
end
--创建输入框函数
function Panel:CreateEditBox(_, LoadValue, SaveValue, DefaultValue)
	local EditBox = CreateFrame("EditBox", UniqueName("EditBox"), self, "InputBoxTemplate")

	if EditBox:IsAutoFocus() then
		EditBox:SetAutoFocus(false)
	end

	EditBox.LoadValue = LoadValue
	EditBox.SaveValue = SaveValue
	EditBox.DefaultValue = DefaultValue
	EditBox.UpdateValue = function(self) self:SetText(self.CurrentValue) end

	EditBox:SetScript("OnEditFocusGained", function(self)
		self:HighlightText()
	end)
	EditBox:SetScript("OnEditFocusLost", function(self)
		self:SetText(self.CurrentValue)
	end)
	EditBox:SetScript("OnEnterPressed", function(self)
		if self:HasFocus() then
			self.CurrentValue = self:GetText()
			self:ClearFocus()
		end
	end)
	EditBox:SetScript("OnEscapePressed", function(self)
		if self:HasFocus() then
			self:SetText(self.CurrentValue)
			self.CurrentValue = self:GetText()
			self:ClearFocus()
		end
	end)

	self.Controls[EditBox:GetName()] = EditBox
	return EditBox
end


--设置面板初始化函数
function Panel:Initialize()

	--插件有关说明文本
	--作者信息
	local Author = self:CreateText(L["|cFFFFC040By:|r |cFF9382C9Aoikaze|r-|cFFFF66FFZeroZone|r-|cFFDE2910CN|r"])
	--反馈和更新链接
	local FeedbackHeading = self:CreateText(L["|cFFFF33CCFeedback & Update: |r"])
	local FeedbackLink = self:CreateEditBox(nil,
		function () return L["Feedback & Update Link"] end,
		function (text) self.CurrentValue = L["Feedback & Update Link"] end,
		L["Feedback & Update Link"])
	FeedbackLink:SetMultiLine(true)
	FeedbackLink:SetHeight(30)
	FeedbackLink:SetWidth(410)

	--启用插件选择框
	local AddonSwitch = self:CreateCheckBox(
		L["Enable |cFFBA55D3SpellWhisper|r"], 
		function() return Config.IsEnable end,
		function(v)	Config.IsEnable = v	end,
		true)

	--仅自身提示模式
	local SelfOnlySwitch = self:CreateCheckBox(
		L["|cFFCC9933Self|r Spell Only"],
		function() return Config.SelfOnly end,
		function(v) Config.SelfOnly = v end,
		false)

	--通告输出模式
	local OutputChannelList = self:CreateDropDown(
		L["Channel: "],
		{"off", L["off"], "say", L["say"], "yell", L["yell"], "party", L["party"], "raid", L["raid"], "Raid_Warning", L["Raid_Warning"], "self", L["self"]},
		function() return Config.OutputChannel end,
		function(v) Config.OutputChannel = v end,
		"raid")

	--滑块 防破控重复刷屏
	local BreakWaitTimeHeading = self:CreateHeading(L["Anti Break Disturbance Time"])
	local BreakWaitTimeSlider = self:CreateSliderBar(
		nil,
		function() return Config.BreakTime end,
		function(v)
			if Config.IsEnable and math.floor(v) ~=0 and not Addon.Frame:IsShown() then
				Addon.Frame:Show()
			end
			Config.BreakTime = math.floor(v)
		end,
		8)
	BreakWaitTimeSlider:SetOrientation("HORIZONTAL")
	BreakWaitTimeSlider:SetHeight(14)
	BreakWaitTimeSlider:SetWidth(160)
	BreakWaitTimeSlider:SetMinMaxValues(0, 20)
	BreakWaitTimeSlider:SetValueStep(1)
	BreakWaitTimeSlider.Low:SetText(L["OFF"])
	BreakWaitTimeSlider.High:SetText(SecondsToTime(20))

	--滑块 防重复打扰延时
	local WaitTimeHeading = self:CreateHeading(L["Anti Disturbance Time"])
	local WaitTimeSlider = self:CreateSliderBar(
		nil,
		function() return Config.WaitTime end, 
		function(v)
			if Config.IsEnable and math.floor(v) ~= 0 and not Addon.Frame:IsShown() then
				Addon.Frame:Show()
			end
			Config.WaitTime = math.floor(v)
		end,
		8)
	WaitTimeSlider:SetOrientation("HORIZONTAL")
	WaitTimeSlider:SetHeight(14)
	WaitTimeSlider:SetWidth(160)
	WaitTimeSlider:SetMinMaxValues(0, 20)
	WaitTimeSlider:SetValueStep(1)
	WaitTimeSlider.Low:SetText(L["OFF"])
	WaitTimeSlider.High:SetText(SecondsToTime(20))

	--密语
	local WhisperHeading = self:CreateHeading(L["Whisper Mode"])
	--启用密语选择框
	local WhisperSwitch = self:CreateCheckBox(
		L["Enable |cFFFFCC33Whisper|r"],
		function() return Config.IsWhisperEnable end,
		function(v) Config.IsWhisperEnable = v end,
		true)
	--控制技能TOT密语提示开关
	local ToTWhisperSwitch = self:CreateCheckBox(
		L["Enable |cFF44CEF6TOT Whisper|r Warning"],
		function() return Config.ToTWarningEnable end,
		function(v) Config.ToTWarningEnable = v end,
		false)

	--战场通告标题
	local BGWarningHeading = self:CreateHeading(L["BG Auto Report"])
	--战场通告选择框
	local BGWarningSwitch = self:CreateCheckBox(
		L["Enable |cFF0099FFBG Auto Report|r"],
		function() return Config.IsBGWarningEnable end,
		function(v) Config.IsBGWarningEnable = v end,
		true)

	--跟随密语设置(标题)
	local FollowHeading = self:CreateHeading(L["Auto Follow Setting"])
	local FollowWhenCombat = self:CreateCheckBox(
		L["Enable |cFFFF6600Combat Follow|r"],
		function() return Config.CombatFollow end,
		function(v) Config.CombatFollow = v end,
		false)
	local StartFollowText = self:CreateText(L["|cFFFF99CCStart Follow|r KeyWords:"])
	local StopFollowText = self:CreateText(L["|cFF999966Stop Follow|r KeyWords:"])
	local CombatFollowSwitchText = self:CreateText(L["|cFFD9B611Combat Follow Mode|r KeyWords:"])
	--跟随密语关键词设置输入框
	local StartFollowKey = self:CreateEditBox(nil,
		function() return Config.StartFollow end,
		function(v) Config.StartFollow = v end,
		"")
	StartFollowKey:SetMultiLine(true)
	StartFollowKey:SetHeight(30)
	StartFollowKey:SetWidth(240)
	--停止跟随密语关键词设置输入框
	local StopFollowKey = self:CreateEditBox(nil,
		function() return Config.StopFollow end,
		function(v) Config.StopFollow = v end,
		"")
	StopFollowKey:SetMultiLine(true)
	StopFollowKey:SetHeight(30)
	StopFollowKey:SetWidth(240)
	--战斗跟随开关关键词设置输入框
	local ChangeCombatFollowModeKey = self:CreateEditBox(nil,
		function() return Config.CombatFollowSwitchKey end,
		function(v) Config.CombatFollowSwitchKey = v end,
		"")
	ChangeCombatFollowModeKey:SetMultiLine(true)
	ChangeCombatFollowModeKey:SetHeight(30)
	ChangeCombatFollowModeKey:SetWidth(240)

	--技能列表标题
	local SpellListHeading = self:CreateHeading(L["Spell List"])
	--添加/删除监控列表类型下拉菜单框
	local DisplaySpellType = self:CreateDropDown(
		L["Spell Monitor: "],
		{"InstantHarm", L["Instant Harm"], "InstantHelp", L["Instant Help"], "CastHarm", L["Cast Harm"], "CastHelp", L["Cast Help"], "SelfBuff", L["Self Buff"], "Other", L["Other"]},
		function() return Config.SelectedSpellType end,
		function(v) Config.SelectedSpellType = v end,
		"Other")
	--列表框
	local SpellListBox = self:CreateText(Addon.CurrentSpellListText)
	--显示列表按钮
	local DisplaySpellButton = self:CreateButton(
		L["Display"],
		function()
			ScrollFrame:ConfigOkay()
			SpellListBox:SetText(Addon.CurrentSpellListText)
			ScrollFrame:ConfigRefresh()
		end)
	--技能列表还原
	local RestoreButton = self:CreateButton(
		L["Restore"],
		function()
			Addon.RunTime = {}
			Addon:UpdateTable(Addon.RunTime, Addon.Default)
			ScrollFrame:ConfigOkay()
			SpellListBox:SetText(Addon.CurrentSpellListText)
			ScrollFrame:ConfigRefresh()
		end)
	--编辑框
	local SpellEditBox = self:CreateEditBox(
		nil,
		function() return "" end,
		function(text) self.CurrentValue = text end,
		"")
	SpellEditBox:SetMultiLine(false)
	SpellEditBox:SetHeight(30)
	SpellEditBox:SetWidth(160)
	--添加按钮
	local AddSpellButton = self:CreateButton(
		L["Add"],
		function()
			if SpellEditBox:GetText() ~= "" then
				Addon.RunTime[Config.SelectedSpellType][SpellEditBox:GetText()] = SpellEditBox:GetText()
			end
			ScrollFrame:ConfigOkay()
			SpellListBox:SetText(Addon.CurrentSpellListText)
			ScrollFrame:ConfigRefresh()
		end)
	local RemoveSpellButton = self:CreateButton(
		L["Remove"],
		function()
			if Addon.RunTime[Config.SelectedSpellType][SpellEditBox:GetText()] then
				Addon.RunTime[Config.SelectedSpellType][SpellEditBox:GetText()] = nil
			end
			ScrollFrame:ConfigOkay()
			SpellListBox:SetText(Addon.CurrentSpellListText)
			ScrollFrame:ConfigRefresh()
		end)

	--控件位置设定
	--作者信息
	Author:SetPoint("LEFT", PanelTitle, "RIGHT", 220, 0)
	--反馈和更新链接
	FeedbackHeading:SetPoint("TOPLEFT", PanelTitle, "BOTTOMLEFT", 30, -30)
	FeedbackLink:SetPoint("LEFT", FeedbackHeading, "RIGHT", 15, 0)
	--插件开关
	AddonSwitch:SetPoint("TOPLEFT", PanelTitle, "BOTTOMLEFT", 0, -80)
	--仅自身提示模式
	SelfOnlySwitch:SetPoint("TOPLEFT", PanelTitle, "BOTTOMLEFT", 200, -80)
	--输出频道
	OutputChannelList:SetPoint("TOPLEFT", AddonSwitch, "BOTTOMLEFT", 110, -20)
	--防打扰破控延时
	BreakWaitTimeHeading:SetPoint("TOPLEFT", AddonSwitch, "BOTTOMLEFT", 0, -75)
	BreakWaitTimeSlider:SetPoint("TOPLEFT", AddonSwitch, "BOTTOMLEFT", BreakWaitTimeHeading:GetStringWidth() + 60, -75)
	--防打扰延时
	WaitTimeHeading:SetPoint("TOPLEFT", AddonSwitch, "BOTTOMLEFT", 0, -120)
	WaitTimeSlider:SetPoint("TOPLEFT", AddonSwitch, "BOTTOMLEFT", WaitTimeHeading:GetStringWidth() + 60, -120)
	--密语通知
	WhisperHeading:SetPoint("TOPLEFT", AddonSwitch, "BOTTOMLEFT", 0, -170)
	WhisperSwitch:SetPoint("TOPLEFT", WhisperHeading, "BOTTOMLEFT", 30, -15)
	ToTWhisperSwitch:SetPoint("TOPLEFT", WhisperHeading, "BOTTOMLEFT", 200, -15)
	--战场自动报告
	BGWarningHeading:SetPoint("TOPLEFT", AddonSwitch, "BOTTOMLEFT", 0, -245)
	BGWarningSwitch:SetPoint("TOPLEFT", BGWarningHeading, "BOTTOMLEFT", 30, -15)
	--密语跟随
	FollowHeading:SetPoint("TOPLEFT", AddonSwitch, "BOTTOMLEFT", 0, -320)
	FollowWhenCombat:SetPoint("LEFT", FollowHeading, "LEFT", 200, 0)
	StartFollowText:SetPoint("TOPLEFT", FollowHeading, "BOTTOMLEFT", 30, -20)
	StopFollowText:SetPoint("TOPLEFT", StartFollowText, "BOTTOMLEFT", 0, -20)
	CombatFollowSwitchText:SetPoint("TOPLEFT", StopFollowText, "BOTTOMLEFT", 0, -20)
	StartFollowKey:SetPoint("LEFT", StartFollowText, "RIGHT", 35, 0)
	StopFollowKey:SetPoint("LEFT", StopFollowText, "RIGHT", 35, 0)
	ChangeCombatFollowModeKey:SetPoint("LEFT", CombatFollowSwitchText, "RIGHT", 35, 0)
	--监控法术类型菜单
	SpellListHeading:SetPoint("TOPLEFT", AddonSwitch, "BOTTOMLEFT", 0, -470)
	DisplaySpellType:SetPoint("TOPLEFT", AddonSwitch, "BOTTOMLEFT", 110, -500)
	DisplaySpellButton:SetPoint("TOPLEFT", AddonSwitch, "BOTTOMLEFT", 280, -500)
	RestoreButton:SetPoint("TOPLEFT", AddonSwitch, "BOTTOMLEFT", 340, -500)
	SpellListBox:SetPoint("TOPLEFT", SpellListHeading, "BOTTOMLEFT", 60, -70)
	SpellEditBox:SetPoint("TOPLEFT", SpellListHeading, "BOTTOMLEFT", 220, -60)
	AddSpellButton:SetPoint("TOPLEFT", SpellEditBox, "BOTTOMLEFT", 10, -15)
	RemoveSpellButton:SetPoint("LEFT", AddSpellButton, "RIGHT", 40, 0)

end

--面板初始化
Panel:Initialize()
Panel:Show()
ScrollFrame.okay = ScrollFrame.ConfigOkay
ScrollFrame.default = ScrollFrame.ConfigDefault
ScrollFrame.refresh = ScrollFrame.ConfigRefresh
ScrollFrame:ConfigRefresh()
ScrollFrame:Show()